from logging import exception
import unittest
from flask import Flask
from unittest import mock
from flask_opsgenie import opsgenie
from flask_opsgenie.entities import OpsgenieAlertParams, AlertType
from flask_opsgenie.opsgenie import make_opsgenie_api_request, raise_manual_alert
from flask_opsgenie.opsgenie import (raise_opsgenie_status_alert, raise_opsgenie_latency_alert,
                                     raise_opsgenie_exception_alert, raise_opsgenie_alert)

class TestOpsgenie(unittest.TestCase):

    def setUp(self):
        self.opsgenie_alert_params = OpsgenieAlertParams(
            opsgenie_token=mock.ANY,
            alert_tags=mock.ANY,
            alert_details={"service_id":"fake_service", "host": "test-host"},
            alert_alias=None,
            alert_status_alias=None,
            alert_latency_alias=None,
            alert_exception_alias=None,
            alert_priority="P4",
            alert_responder=mock.ANY,
            opsgenie_api_base="https://dummy-opsgenie.com"
        )

    @mock.patch('flask_opsgenie.opsgenie.requests.post')
    @mock.patch('flask_opsgenie.opsgenie.requests.get')
    def test_make_opsgenie_api_request(self, mock_get, mock_post):

        _ = make_opsgenie_api_request(http_verb="get", url=mock.ANY, payload=mock.ANY, opsgenie_token=mock.ANY)
        self.assertEqual(mock_get.call_count, 1)
        _ = make_opsgenie_api_request(http_verb="post", url=mock.ANY, payload=mock.ANY, opsgenie_token=mock.ANY)
        self.assertEqual(mock_post.call_count, 1)

    @mock.patch('flask_opsgenie.opsgenie.make_opsgenie_api_request')
    def test_raise_opsgenie_status_alert(self, mock_opsgenie_api_request):

        app = Flask(__name__)
        with app.test_client() as client:
            rv = client.get('/test/og_params')
            _ = raise_opsgenie_status_alert(alert_status_code=500, alert_status_class=None, opsgenie_alert_params=self.opsgenie_alert_params, response_status_code=500, extra_props={'test-key': 'test-value'})
            self.assertEqual(self.opsgenie_alert_params.alert_details['endpoint'], '/test/og_params')
            self.assertEqual(self.opsgenie_alert_params.alert_details['method'], "GET")
            self.assertEqual(self.opsgenie_alert_params.alert_details['url'], 'http://localhost/test/og_params')
            self.assertEqual(self.opsgenie_alert_params.alert_details['status_code'], 500)
            self.assertEqual(self.opsgenie_alert_params.alert_details.get('status_class'), None)
            self.assertEqual(self.opsgenie_alert_params.alert_details.get('test-key'), 'test-value')
            self.assertEqual(self.opsgenie_alert_params.alert_status_alias, 'fake_service-/test/og_params-500')
            self.assertEqual(mock_opsgenie_api_request.call_count, 1)
            mock_opsgenie_api_request.assert_called_with(
                http_verb="post",
                url=f'{self.opsgenie_alert_params.opsgenie_api_base}/v2/alerts',
                payload=mock.ANY,
                opsgenie_token=self.opsgenie_alert_params.opsgenie_token
            )


    @mock.patch('flask_opsgenie.opsgenie.make_opsgenie_api_request')
    def test_raise_opsgenie_latency_alert(self, mock_opsgenie_api_request):

        app = Flask(__name__)
        with app.test_client() as client:
            rv = client.get('/test/og_params')
            _ = raise_opsgenie_latency_alert(elapsed_time=2000, alert_status_code=500, opsgenie_alert_params=self.opsgenie_alert_params, extra_props={'test-key': 'test-value'})
            self.assertEqual(self.opsgenie_alert_params.alert_details['endpoint'], '/test/og_params')
            self.assertEqual(self.opsgenie_alert_params.alert_details['method'], "GET")
            self.assertEqual(self.opsgenie_alert_params.alert_details['url'], 'http://localhost/test/og_params')
            self.assertEqual(self.opsgenie_alert_params.alert_details['status_code'], 500)
            self.assertEqual(self.opsgenie_alert_params.alert_details.get('status_class'), None)
            self.assertEqual(self.opsgenie_alert_params.alert_details.get('test-key'), 'test-value')
            self.assertEqual(self.opsgenie_alert_params.alert_latency_alias, 'fake_service-response-latency-alert')
            self.assertEqual(mock_opsgenie_api_request.call_count, 1)
            mock_opsgenie_api_request.assert_called_with(
                http_verb="post",
                url=f'{self.opsgenie_alert_params.opsgenie_api_base}/v2/alerts',
                payload=mock.ANY,
                opsgenie_token=self.opsgenie_alert_params.opsgenie_token
            )

    @mock.patch('flask_opsgenie.opsgenie.make_opsgenie_api_request')
    @mock.patch('flask_opsgenie.opsgenie.traceback')
    @mock.patch('flask_opsgenie.opsgenie.Exception')
    def test_raise_opsgenie_exception_alert(self, mock_exception, mock_traceback, mock_opsgenie_api_request):

        mock_exception.__traceback__ = ""
        mock_traceback.format_exception.return_value = ""
        dummy_exception_class = "MagicMock"

        app = Flask(__name__)
        with app.test_client() as client:
            rv = client.get('/test/og_params')
            _ = raise_opsgenie_exception_alert(exception=mock_exception, opsgenie_alert_params=self.opsgenie_alert_params, extra_props={'test-key': 'test-value'})
            self.assertEqual(self.opsgenie_alert_params.alert_details['endpoint'], '/test/og_params')
            self.assertEqual(self.opsgenie_alert_params.alert_details['method'], "GET")
            self.assertEqual(self.opsgenie_alert_params.alert_details['url'], 'http://localhost/test/og_params')
            self.assertEqual(self.opsgenie_alert_params.alert_details['exception'], str(mock_exception))
            self.assertEqual(self.opsgenie_alert_params.alert_details.get('test-key'), 'test-value')
            self.assertEqual(self.opsgenie_alert_params.alert_exception_alias, f'fake_service-/test/og_params-{dummy_exception_class}')
            self.assertEqual(mock_opsgenie_api_request.call_count, 1)
            mock_opsgenie_api_request.assert_called_with(
                http_verb="post",
                url=f'{self.opsgenie_alert_params.opsgenie_api_base}/v2/alerts',
                payload=mock.ANY,
                opsgenie_token=self.opsgenie_alert_params.opsgenie_token
            )

    @mock.patch('flask_opsgenie.opsgenie.make_opsgenie_api_request')
    @mock.patch('flask_opsgenie.opsgenie.traceback')
    @mock.patch('flask_opsgenie.opsgenie.Exception')
    def test_raise_manual_alert(self, mock_exception, mock_traceback, mock_opsgenie_api_request):

        mock_exception.__traceback__ = ""
        mock_traceback.format_exception.return_value = ""
        test_func_name = "test_func_name"
        dummy_exception_class = "MagicMock"

        app = Flask(__name__)
        with app.test_client() as client:
            rv = client.get('/test/og_params')
            _ = raise_manual_alert(exception=mock_exception, func_name=test_func_name, opsgenie_alert_params=self.opsgenie_alert_params, extra_props={'test-key': 'test-value'})
            self.assertEqual(self.opsgenie_alert_params.alert_details['exception'], str(mock_exception))
            self.assertEqual(self.opsgenie_alert_params.alert_details['Traceback'], "")
            self.assertEqual(self.opsgenie_alert_params.alert_details.get('test-key'), 'test-value')
            self.assertEqual(self.opsgenie_alert_params.alert_alias, f'fake_service-{test_func_name}-{dummy_exception_class}')
            self.assertEqual(mock_opsgenie_api_request.call_count, 1)
            mock_opsgenie_api_request.assert_called_with(
                http_verb="post",
                url=f'{self.opsgenie_alert_params.opsgenie_api_base}/v2/alerts',
                payload=mock.ANY,
                opsgenie_token=self.opsgenie_alert_params.opsgenie_token
            )

    @mock.patch('flask_opsgenie.opsgenie.raise_opsgenie_status_alert')
    @mock.patch('flask_opsgenie.opsgenie.raise_opsgenie_latency_alert')
    @mock.patch('flask_opsgenie.opsgenie.raise_opsgenie_exception_alert')
    @mock.patch('flask_opsgenie.opsgenie.raise_manual_alert')
    def test_raise_opsgenie_alert(self, mock_raise_manual_alert, mock_opsgenie_exception_alert, mock_opsgenie_latency_alert,
                                  mock_opsgenie_status_alert):

        test_exception = mock.ANY
        test_status_code = 500
        test_status_class = "5XX"
        test_elapsed_time = 2000
        test_manual_exception = "test_exception"
        test_func_name = "test_func_name"

        _ = raise_opsgenie_alert(alert_type=AlertType.STATUS_ALERT, alert_status_code=test_status_code,
                                 elapsed_time = None, opsgenie_alert_params=self.opsgenie_alert_params, response_status_code=test_status_code)
        self.assertEqual(mock_opsgenie_status_alert.call_count, 1)
        mock_opsgenie_status_alert.assert_called_with(
            alert_status_code=test_status_code,
            opsgenie_alert_params=self.opsgenie_alert_params,
            response_status_code=test_status_code,
            extra_props=mock.ANY
        )
        _ = raise_opsgenie_alert(alert_type=AlertType.STATUS_ALERT, alert_status_class=test_status_class,
                                 elapsed_time = None, opsgenie_alert_params=self.opsgenie_alert_params, response_status_code=test_status_code)
        self.assertEqual(mock_opsgenie_status_alert.call_count, 2)
        mock_opsgenie_status_alert.assert_called_with(
            alert_status_class=test_status_class,
            opsgenie_alert_params=self.opsgenie_alert_params,
            response_status_code=test_status_code,
            extra_props=mock.ANY
        )
        _ = raise_opsgenie_alert(alert_type=AlertType.LATENCY_ALERT, alert_status_code=test_status_code,
                                 elapsed_time=test_elapsed_time, opsgenie_alert_params=self.opsgenie_alert_params)
        self.assertEqual(mock_opsgenie_latency_alert.call_count, 1)
        mock_opsgenie_latency_alert.assert_called_with(
            elapsed_time=test_elapsed_time,
            alert_status_code=test_status_code,
            opsgenie_alert_params=self.opsgenie_alert_params,
            extra_props=mock.ANY
        )
        _ = raise_opsgenie_alert(alert_type=AlertType.EXCEPTION, exception=test_exception, opsgenie_alert_params=self.opsgenie_alert_params)
        self.assertEqual(mock_opsgenie_exception_alert.call_count, 1)
        mock_opsgenie_exception_alert.assert_called_with(
            exception=test_exception,
            opsgenie_alert_params=self.opsgenie_alert_params,
            extra_props=mock.ANY
        )

        _ = raise_opsgenie_alert(alert_type=AlertType.MANUAL, exception=test_manual_exception, func_name=test_func_name, opsgenie_alert_params=self.opsgenie_alert_params)
        self.assertEqual(mock_raise_manual_alert.call_count, 1)
        mock_raise_manual_alert.assert_called_with(
            exception=test_manual_exception,
            opsgenie_alert_params=self.opsgenie_alert_params,
            func_name=test_func_name,
            extra_props=mock.ANY
        )



import os
import re
import socket
import time
from typing import List, Optional, Dict

from flask import Flask, request, Response, g, _app_ctx_stack as stack
from flask_opsgenie.opsgenie import raise_opsgenie_alert
from flask_opsgenie.entities import AlertType
from flask_opsgenie.entities import OpsgenieAlertParams
from http import HTTPStatus

CONFIG_ALERT_STATUS_CODES = "ALERT_STATUS_CODES"
CONFIG_ALERT_STATUS_CLASSES = "ALERT_STATUS_CLASSES"
CONFIG_MONITORED_ENDPOINTS = "MONITORED_ENDPOINTS"
CONFIG_IGNORED_ENDPOINTS = "IGNORED_ENDPOINTS"
CONFIG_THRESHOLD_RESPONSE_TIME = "THRESHOLD_RESPONSE_TIME"
CONFIG_RESPONSE_TIME_MONITORED_ENDPOINTS = "RESPONSE_TIME_MONITORED_ENDPOINTS"
CONFIG_OPSGENIE_TOKEN = "OPSGENIE_TOKEN"
CONFIG_ALERT_TAGS = "ALERT_TAGS"
CONFIG_ALERT_DETAILS = "ALERT_DETAILS"
CONFIG_ALERT_PRIORITY = "ALERT_PRIORITY"
CONFIG_ALERT_ALIAS = "ALERT_ALIAS"
CONFIG_ALERT_STATUS_ALIAS = "ALERT_STATUS_ALIAS"
CONFIG_ALERT_LATENCY_ALIAS = "ALERT_LATENCY_ALIAS"
CONFIG_ALERT_EXCEPTION_ALIAS = "ALERT_EXCEPTION_ALIAS"
CONFIG_RESPONDER = "RESPONDER"
CONFIG_OPSGENIE_API_BASE = "OPSGENIE_API_BASE"
CONFIG_SERVICE_ID = "SERVICE_ID"
CONFIG_ALERT_EXCEPTION = "ALERT_EXCEPTION"
CONFIG_NO_TRACEBACK = "NO_TRACEBACK"
CONFIG_FORWARDED_HEADER_KEYS = "FORWARDED_HEADER_KEYS"
OPSGENIE_API_BASE_US = "https://api.opsgenie.com"


class FlaskOpsgenie(object):

    def __init__(self, app: Optional[Flask]):

        self.app = app
        self._host = None
        self._alert_status_codes = None
        self._alert_status_classes = None
        self._monitored_endpoints = None
        self._ignored_endpoints = None
        self._threshold_response_time = None
        self._response_time_monitored_endpoints = None
        self._opsgenie_token = None
        self._alert_tags = None
        self._alert_details = None
        self._alert_priority = None
        self._alert_alias = None
        self._alert_status_alias = None
        self._alert_latency_alias = None
        self._alert_exception_alias = None
        self._responder = None
        self._opsgenie_api_base = None
        self._service_id = None
        self._forwarded_header_keys = None
        self._request_headers = None

        if app is not None:
            self.init_app(app)

    def init_app(self, app: Flask):

        self._alert_status_codes =  [int(x) for x in app.config.get(CONFIG_ALERT_STATUS_CODES, [])]
        self._alert_status_classes = app.config.get(CONFIG_ALERT_STATUS_CLASSES)
        self._monitored_endpoints = app.config.get(CONFIG_MONITORED_ENDPOINTS)
        self._ignored_endpoints = app.config.get(CONFIG_IGNORED_ENDPOINTS)
        self._threshold_response_time = app.config.get(CONFIG_THRESHOLD_RESPONSE_TIME)
        self._response_time_monitored_endpoints = app.config.get(CONFIG_RESPONSE_TIME_MONITORED_ENDPOINTS)
        self._opsgenie_token = app.config.get(CONFIG_OPSGENIE_TOKEN)
        self._alert_tags = app.config.get(CONFIG_ALERT_TAGS)
        self._alert_details = app.config.get(CONFIG_ALERT_DETAILS, {})
        self._alert_alias = app.config.get(CONFIG_ALERT_ALIAS)
        self._alert_status_alias = app.config.get(CONFIG_ALERT_STATUS_ALIAS, self._alert_alias)
        self._alert_latency_alias = app.config.get(CONFIG_ALERT_LATENCY_ALIAS, self._alert_alias)
        self._alert_exception_alias = app.config.get(CONFIG_ALERT_EXCEPTION_ALIAS, self._alert_alias)
        self._alert_priority = app.config.get(CONFIG_ALERT_PRIORITY, "P4")
        self._responder = app.config.get(CONFIG_RESPONDER)
        self._opsgenie_api_base = app.config.get(CONFIG_OPSGENIE_API_BASE, OPSGENIE_API_BASE_US)
        self._service_id = app.config.get(CONFIG_SERVICE_ID)
        self._alert_exception = app.config.get(CONFIG_ALERT_EXCEPTION, False)
        self._no_traceback = app.config.get(CONFIG_NO_TRACEBACK, False)
        self._forwarded_header_keys = app.config.get(CONFIG_FORWARDED_HEADER_KEYS, False)
        self._host = socket.gethostname()

        # pre-process status_class list if present
        if self._alert_status_classes:
            self._alert_status_classes = [s_class.upper() for s_class in self._alert_status_classes]

        # add host and service to alert details as well
        self._alert_details["host"] = self._host
        self._alert_details["service_id"] = self._service_id

        app.before_request(self._before_request)
        app.after_request(self._after_request)

    def opsgenie_params_util(self) -> OpsgenieAlertParams:

        return OpsgenieAlertParams(
            opsgenie_token=self._opsgenie_token,
            alert_tags=self._alert_tags,
            alert_details=self._alert_details,
            alert_alias=self._alert_alias,
            alert_status_alias=self._alert_status_alias,
            alert_latency_alias=self._alert_latency_alias,
            alert_exception_alias=self._alert_exception_alias,
            no_traceback = self._no_traceback,
            alert_priority=self._alert_priority,
            alert_responder=self._responder,
            opsgenie_api_base=self._opsgenie_api_base
        )

    def _get_status_class(self, status_code: int) -> str:
        return str(status_code)[0] + "XX"

    def _path_present(self, endpoint:str, endpoint_patterns:List[str]):
        match_results = [False if re.match(pattern, endpoint) == None else True for pattern in endpoint_patterns]
        return any(match_results)

    def _before_request(self):
        self._request_headers = request.headers
        ctx = stack.top
        ctx._flask_request_begin_at = time.time()

    def _after_request(self, response: Response):
        ctx = stack.top
        elapsed_time = (time.time() - ctx._flask_request_begin_at) * 1000

        status_code = response.status_code

        # get value if status code is of type enum HTTPStatus
        status_code = status_code.value if type(status_code) == HTTPStatus else status_code
        status_class = self._get_status_class(status_code)
        endpoint = request.path

        # Fetching Request header values
        extra_props = self._extract_request_header()

        if (self._alert_status_codes and status_code in self._alert_status_codes) or \
                (self._alert_status_classes and status_class in self._alert_status_classes):
            if (self._monitored_endpoints and self._path_present(endpoint, self._monitored_endpoints)) or \
                    (not self._monitored_endpoints and not(self._ignored_endpoints and self._path_present(endpoint, self._ignored_endpoints))):
                if self._alert_status_codes and status_code in self._alert_status_codes:

                    raise_opsgenie_alert(AlertType.STATUS_ALERT, alert_status_code=status_code,
                                         opsgenie_alert_params=self.opsgenie_params_util(), response_status_code=status_code,
                                         extra_props=extra_props)

                elif self._alert_status_classes and status_class in self._alert_status_classes:
                    raise_opsgenie_alert(AlertType.STATUS_ALERT, alert_status_class=status_class,
                                         opsgenie_alert_params=self.opsgenie_params_util(), response_status_code=status_code,
                                         extra_props=extra_props)

        if self._threshold_response_time and self._response_time_monitored_endpoints and \
                self._path_present(endpoint, self._response_time_monitored_endpoints) \
                and elapsed_time > self._threshold_response_time:

            raise_opsgenie_alert(AlertType.LATENCY_ALERT, elapsed_time=elapsed_time, alert_status_code=status_code,
                                 opsgenie_alert_params=self.opsgenie_params_util(), extra_props=extra_props)

        return response

    def raise_exception_alert(self, alert_type:AlertType = None, exception=None, func_name:str=None,
                              extra_props: Dict[str,str] = {}, alert_priority: str = ''):

        # Fetching Request header values
        header_details = self._extract_request_header()
        extra_props = {**extra_props, **header_details}

        raise_opsgenie_alert(alert_type=alert_type, exception=exception, func_name=func_name, opsgenie_alert_params=self.opsgenie_params_util(),
                             extra_props=extra_props, alert_priority=alert_priority)

    def raise_gevent_exception_alert(self, greenlet):
        try:
            greenlet.get()
        except Exception as e:
            # Fetching Request header values
            extra_props = self._extract_request_header()
            self.raise_exception_alert(alert_type=AlertType.MANUAL, exception=e, func_name="gevent", extra_props=extra_props)

    def gevent_exception_callback(self, g):
        g.link_exception(self.raise_gevent_exception_alert)


    def _extract_request_header(self):
        extra_props = {}
        if self._forwarded_header_keys:
            for header_key in self._forwarded_header_keys:
                extra_props[header_key] = self._request_headers.get(header_key, None)

        return extra_props


import logging
import requests
import sys
from requests import HTTPError
import traceback
from typing import Any, Dict, Optional
from flask import request
from flask_opsgenie.entities import AlertType, OpsgenieAlertParams, AlertPriority

logger = logging.getLogger(__name__)


def make_opsgenie_api_request(http_verb:str="get", url:str=None, payload:Dict[str, Any]=None, opsgenie_token:str=None):

    headers = {
        "Content-Type": "application/json",
        "Authorization": f'GenieKey {opsgenie_token}'
    }

    response = getattr(requests, http_verb)(
        url, headers=headers, json=payload
    )
    response.raise_for_status()


def raise_opsgenie_status_alert(alert_status_code:Optional[str] = None, alert_status_class:Optional[str] = None,
                                opsgenie_alert_params:OpsgenieAlertParams=None, response_status_code: int=None, extra_props:Dict[str, str] = {}):

    endpoint = request.path
    url = request.url
    method = request.method
    summary = ""
    description = ""

    # add url info into details
    opsgenie_alert_params.alert_details["endpoint"] = endpoint
    opsgenie_alert_params.alert_details["url"] = url
    opsgenie_alert_params.alert_details["method"] = method

    # Add header values to alert_details
    opsgenie_alert_params.alert_details = {**opsgenie_alert_params.alert_details, **extra_props}

    # update the status code/class as well in details, will help in searching
    if alert_status_code:
        opsgenie_alert_params.alert_details["status_code"] = alert_status_code
    else:
        opsgenie_alert_params.alert_details["status_class"] = alert_status_class

    # update alias if not set
    # Alias: serviceID-<path>-<status-code>-ALERT_STATUS_ALIAS
    alias = f'{opsgenie_alert_params.alert_details["service_id"]}-{request.path}-{response_status_code}'
    if opsgenie_alert_params.alert_status_alias:
        opsgenie_alert_params.alert_status_alias = alias + "-" + opsgenie_alert_params.alert_status_alias
    else:
        opsgenie_alert_params.alert_status_alias =  alias

    if alert_status_code:
        summary = f'{endpoint} returned unaccepted status code : {alert_status_code} | Alert generated from {opsgenie_alert_params.alert_details["service_id"]}'
        description = f'{endpoint} returned status code : {alert_status_code}. Complete URL : {url} called with method ' \
                      f'{method}. Endpoint served by service : {opsgenie_alert_params.alert_details["service_id"]} on host: ' \
                      f'{opsgenie_alert_params.alert_details["host"]}'
    if alert_status_class:
        summary = f'{endpoint} returned unaccepted status class : {alert_status_class} | Alert generated from flask'
        description = f'{endpoint} returned status code from class : {alert_status_class}. Complete URL : {url} called with method ' \
                      f'{method}. Endpoint served by service : {opsgenie_alert_params.alert_details["service_id"]} on host: ' \
                      f'{opsgenie_alert_params.alert_details["host"]}'

    payload = {
        "message": summary,
        "description": description,
        "alias": opsgenie_alert_params.alert_status_alias,
        "tags": opsgenie_alert_params.alert_tags,
        "details": opsgenie_alert_params.alert_details,
        "priority": opsgenie_alert_params.alert_priority.value,
    }

    # add responders if present
    if opsgenie_alert_params.alert_responder:
        payload["responders"] = opsgenie_alert_params.alert_responder

    # Now we are all set to make the alert api call to opsgenie
    try:
        make_opsgenie_api_request(
            http_verb="post", url=f'{opsgenie_alert_params.opsgenie_api_base}/v2/alerts', payload=payload,
            opsgenie_token=opsgenie_alert_params.opsgenie_token
        )
    except HTTPError as e:
        logger.exception(e)


def raise_opsgenie_latency_alert(elapsed_time:int, alert_status_code:int, opsgenie_alert_params:OpsgenieAlertParams=None, extra_props:Dict[str, str] = {}):
    endpoint = request.path
    url = request.url
    method = request.method

    # add url info into details
    opsgenie_alert_params.alert_details["endpoint"] = endpoint
    opsgenie_alert_params.alert_details["url"] = url
    opsgenie_alert_params.alert_details["method"] = method
    opsgenie_alert_params.alert_details["status_code"] = alert_status_code

    # Add header values to alert_details
    opsgenie_alert_params.alert_details = {**opsgenie_alert_params.alert_details, **extra_props}

    # update alias if not set
    if not opsgenie_alert_params.alert_latency_alias:
        opsgenie_alert_params.alert_latency_alias = f'{opsgenie_alert_params.alert_details["service_id"]}-response-latency-alert'

    summary = f'{endpoint} showed unexpected response time : {elapsed_time}ms | Alert generated from flask'
    description = f'{endpoint} showed unexpected response time : {elapsed_time}ms. Complete URL : {url} called with method ' \
                    f'{method}. Endpoint served by service : {opsgenie_alert_params.alert_details["service_id"]} on host: ' \
                    f'{opsgenie_alert_params.alert_details["host"]}'

    payload = {
        "message": summary,
        "description": description,
        "alias": opsgenie_alert_params.alert_latency_alias,
        "tags": opsgenie_alert_params.alert_tags,
        "details": opsgenie_alert_params.alert_details,
        "priority": opsgenie_alert_params.alert_priority.value,
    }

    # add responders if present
    if opsgenie_alert_params.alert_responder:
        payload["responders"] = opsgenie_alert_params.alert_responder

    # Now we are all set to make the alert api call to opsgenie
    try:
        make_opsgenie_api_request(
            http_verb="post", url=f'{opsgenie_alert_params.opsgenie_api_base}/v2/alerts', payload=payload,
            opsgenie_token=opsgenie_alert_params.opsgenie_token
        )
    except HTTPError as e:
        logger.exception(e)


def raise_opsgenie_exception_alert(exception:Exception=None, opsgenie_alert_params:OpsgenieAlertParams=None,
                                   extra_props: Dict[str,str] = {}):

    endpoint = request.path
    url = request.url
    method = request.method

    # add url info into details
    opsgenie_alert_params.alert_details["endpoint"] = endpoint
    opsgenie_alert_params.alert_details["url"] = url
    opsgenie_alert_params.alert_details["method"] = method
    opsgenie_alert_params.alert_details["exception"] = str(exception)

    # merge existing details dict with extra_props
    opsgenie_alert_params.alert_details = {**opsgenie_alert_params.alert_details, **extra_props}

    # update alias if not set
    # Alias: serviceid-<path>-<exception-name>-ALERT_EXCEPTION_ALIAS
    alias = f'{opsgenie_alert_params.alert_details["service_id"]}-{endpoint}-{exception.__class__.__name__}'
    if opsgenie_alert_params.alert_exception_alias:
        opsgenie_alert_params.alert_exception_alias = alias + "-" + opsgenie_alert_params.alert_exception_alias
    else:
        opsgenie_alert_params.alert_exception_alias = alias

    summary = f'{endpoint} threw exception : {str(exception)} | Alert generated from {opsgenie_alert_params.alert_details["service_id"]}'
    description = f'{endpoint} has thrown exception : {str(exception)}. Complete URL : {url} called with method ' \
                    f'{method}. Endpoint served by service : {opsgenie_alert_params.alert_details["service_id"]} on host: ' \
                    f'{opsgenie_alert_params.alert_details["host"]}.'

    payload = {
        "message": summary,
        "description": description,
        "alias": opsgenie_alert_params.alert_exception_alias,
        "tags": opsgenie_alert_params.alert_tags,
        "details": opsgenie_alert_params.alert_details,
        "priority": opsgenie_alert_params.alert_priority.value,
    }

    # add responders if present
    if opsgenie_alert_params.alert_responder:
        payload["responders"] = opsgenie_alert_params.alert_responder

    # if traceback is not silenced then add traceback
    if not opsgenie_alert_params.no_traceback:
        if sys.version_info.major == 3 and sys.version_info.minor >= 10:
            traceback_str = "".join(traceback.format_exception(exception))
        else:
            traceback_str = "".join(traceback.format_exception(etype=type(exception),
                                                               value=exception,
                                                               tb=exception.__traceback__))
        payload["description"] = f'{description} {traceback_str}'
        payload["details"]["Traceback"] = traceback_str

    # Now we are all set to make the alert api call to opsgenie
    try:
        make_opsgenie_api_request(
            http_verb="post", url=f'{opsgenie_alert_params.opsgenie_api_base}/v2/alerts', payload=payload,
            opsgenie_token=opsgenie_alert_params.opsgenie_token
        )
    except HTTPError as e:
        logger.exception(e)


def raise_manual_alert(exception:Exception=None, func_name:str=None, opsgenie_alert_params:OpsgenieAlertParams=None,
                       extra_props: Dict[str,str] = {}):

    if sys.version_info.major == 3 and sys.version_info.minor >= 10:
        trace_back = "".join(traceback.format_exception(exception))
    else:
        trace_back = "".join(traceback.format_exception(etype=type(exception), value=exception, tb=exception.__traceback__))

    exception_str = str(exception).replace(' ', '_')
    # add url info into details
    opsgenie_alert_params.alert_details["exception"] = str(exception)
    opsgenie_alert_params.alert_details["Traceback"] = trace_back

    # merge existing details dict with extra_props
    opsgenie_alert_params.alert_details = {**opsgenie_alert_params.alert_details, **extra_props}

    # update alias if not set
    # Alias: serviceid-funcname-exception-name-ALERT_ALIAS
    alias = f'{opsgenie_alert_params.alert_details["service_id"]}-{func_name}-{exception.__class__.__name__}'
    if opsgenie_alert_params.alert_alias:
        opsgenie_alert_params.alert_alias = alias + "-" + opsgenie_alert_params.alert_alias
    else:
        opsgenie_alert_params.alert_alias = alias

    summary = f'{func_name} threw exception : {exception_str} | Alert generated from {opsgenie_alert_params.alert_details["service_id"]}'
    description = f'{func_name} has thrown exception : {exception_str}. Complete stack trace : {trace_back}'

    payload = {
        "message": summary,
        "description": description,
        "alias": opsgenie_alert_params.alert_alias,
        "tags": opsgenie_alert_params.alert_tags,
        "details": opsgenie_alert_params.alert_details,
        "priority": opsgenie_alert_params.alert_priority.value,
    }

    # add responders if present
    if opsgenie_alert_params.alert_responder:
        payload["responders"] = opsgenie_alert_params.alert_responder

    # Now we are all set to make the alert api call to opsgenie
    try:
        make_opsgenie_api_request(
            http_verb="post", url=f'{opsgenie_alert_params.opsgenie_api_base}/v2/alerts', payload=payload,
            opsgenie_token=opsgenie_alert_params.opsgenie_token
        )
    except HTTPError as e:
        logger.exception(e)



def raise_opsgenie_alert(alert_type:AlertType = None, alert_status_code:Optional[int] = None, \
                         alert_status_class:Optional[str] = None, elapsed_time:Optional[int] = None,
                         exception=None, opsgenie_alert_params:OpsgenieAlertParams=None, response_status_code: int=None,
                         func_name:str=None, extra_props: Dict[str,str] = {}, alert_priority: str = ''):

    # handle overrides
    if alert_priority != '':
        try:
            opsgenie_alert_params.alert_priority = AlertPriority(alert_priority)
        except ValueError as e:
            logger.exception(e)

    if alert_type == AlertType.STATUS_ALERT:
        if alert_status_code:
            raise_opsgenie_status_alert(alert_status_code=alert_status_code, opsgenie_alert_params=opsgenie_alert_params,
                                        response_status_code=response_status_code, extra_props=extra_props)
        elif alert_status_class:
            raise_opsgenie_status_alert(alert_status_class=alert_status_class, opsgenie_alert_params=opsgenie_alert_params,
                                        response_status_code=response_status_code, extra_props=extra_props)

    if alert_type == AlertType.LATENCY_ALERT:
        raise_opsgenie_latency_alert(elapsed_time=elapsed_time, alert_status_code=alert_status_code,
                                     opsgenie_alert_params=opsgenie_alert_params, extra_props=extra_props)

    if alert_type == AlertType.EXCEPTION:
        raise_opsgenie_exception_alert(exception=exception, opsgenie_alert_params=opsgenie_alert_params,
                                       extra_props=extra_props)

    if alert_type == AlertType.MANUAL:
        raise_manual_alert(exception=exception, func_name=func_name, opsgenie_alert_params=opsgenie_alert_params,
                           extra_props=extra_props)




"""
Flask-Opsgenie
---------------

An easy to use Opsgenie extension for flask. Allows user
to raise an opsgenie alert on unwanted response status code,
increased response latency and on unhandled exception thrown
by routes.
With flask-opsgenie, you will no more have to add alerting
logic to your code manually, rather all you need to do is configure
this extension with different alert conditions and attributes.
"""

from setuptools import find_packages, setup

setup(
    name="Flask-Opsgenie",
    url="https://github.com/djmgit/flask-opsgenie",
    license="",
    author="Deepjyoti Mondal",
    description="Opsgenie extension for Flask",
    download_url="https://github.com/djmgit/flask-opsgenie/archive/refs/tags/v0.4.2.tar.gz",
    long_description=__doc__,
    zip_safe=False,
    keywords = ['Alerting', 'flask', 'web', 'Reliability', 'DevOps'],
    platforms="any",
    packages=find_packages(),
    install_requires=[
        "requests>=2.0.0",
        "Flask>=1.1.2"
    ],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: System :: Monitoring',
    ],
    version='0.4.2'
)
